*! version 1.0.0

program nchooseki, rclass
      version 15
      args n k nothing 

      confirm integer number `n'
      confirm integer number `k'
      if ("`nothing'" != "") { 
           display as err "`nothing' found where nothing expected"
           exit 198
      }

      mata: st_numscalar("r(result)", n_choose_k(`n', `k'))
      return scalar comb = r(result)
      display as txt "  `n' choose `k' = " as res r(result) 
end


version 15
set matastrict on

mata:

/*    _
      N = n_choose_k(n, k)
                     -  -

              N = n! / ( (n-k)! k! )

      Formula evaluated using canceling to increase domain.
*/

real scalar n_choose_k(real scalar n, real scalar k)
{
      return( n-k > k ? 
              nfactorial_over_kfactorial(n, n-k) / 
              nfactorial_over_kfactorial(k, 1)
              :
              nfactorial_over_kfactorial(n, k) / 
              nfactorial_over_kfactorial(n-k, 1)
            )
}

/*    ______
      result = nfactorial_over_kfactorial(n, k)
                                          -  -
                
      Returns n!/k! calculated as n*(n-1)*...*(k+1).
      Note: 0! = 1.
*/


real scalar nfactorial_over_kfactorial(real scalar n, real scalar k)
{
      real scalar     result, i

      if (n<0 | n>1.0x+35 | n!=trunc(n))  return(.)
      if (k<0 | k>1.0x+35 | k!=trunc(k))  return(.)

      result = 1 
      for (i=n; i>k; --i) result = result*i

      return(result)
}
end
