*! SpMat version 2.0.0

version 15
set matastrict on

/*
    This is file spmat2.mata.

    Differences between spmat.mata and this file.

        This file was named spmat11.mata in book0. !!DELETE

        spmat2.mata has `RV' for declaring real vector.

        spmat2.mata Has new function `Sp'::cols_of_row().

        spmat2.mata has better Code for `Sp'::R_SxS().

    The purpose of spmat2.mata is to evaluate the performance 
    of the new logic for multiplying sparse matrices, R_SxS().

*/
        
        


/* -------------------------------------------------------------------- */
                                        /* macro'd types                */


local Sp                SpMat

local RS                real scalar
local RR                real rowvector
local RC                real colvector
local RV                real vector
local RM                real matrix
local Sparse            class `Sp' scalar
local Regular           real matrix
local AA                class AssociativeArray scalar

local boolean           real scalar
      local True  1
      local False 0


mata:
/* -------------------------------------------------------------------- */
                                        /* class definition                */
class `Sp'
{
    public:
      void            setup() 

      `RS'            rows(), cols()

      `RS'            get()
      void            put(), replace()

      `RM'            Get()
      void            Put(), Replace()

      //      returns = A*B         R = Regular matrix
      //             \  | |         S = Sparse  matrix
      `Regular'       R_SxR()
      `Sparse'        S_SxR()
      `Regular'       R_RxS()
      `Sparse'        S_RxS()
      `Regular'       R_SxS()
      `Sparse'        S_SxS()
      void            S_RxR()

      //      returns = A+B         R = Regular matrix
      //             \  | |         S = Sparse  matrix
      `Regular'       R_SpS()         
      `Sparse'        S_SpS() 
      `Regular'       R_SpR()
      `Sparse'        S_SpR()
      void            S_RpR()

    private:
      `RM'            Get2(),     Get4()
      void            Put3(),     Put5()
      void            Replace3(), Replace5()

      void            set_row(),  set_col()

      pointer(`RV') vector cols_of_row()

      `AA'            AA
      `RS'            rows, cols
}


/* -------------------------------------------------------------------- */

void `Sp'::setup(`RS' rows, `RS' cols)
{
      AA.reinit("real", 2)
      AA.notfound(0)

      this.rows = rows
      this.cols = cols
}


`RS' `Sp'::rows()                 return(rows)
`RS' `Sp'::cols()                 return(cols)


/* -------------------------------------------------------------------- */
                
/* 
    Low-level access to sparse matrix:

            S.get(i, j)
            S.put(i, j, v)
            S.replace(i, j, v)

    These are the ONLY routines that access AssociativeArray AA.
    Other functions access sparse matrices using these functions.
*/

`RS' `Sp'::get(`RS' i, `RS' j)  return( AA.get((i,j)) )


void `Sp'::put(`RS' i, `RS' j, `RS' v)
{
      if (v) AA.put((i,j), v)
}

void `Sp'::replace(`RS' i, `RS' j, `RS' v)
{
      if (v)    AA.put( i, j, v)
      else   AA.remove((i, j))
}



/* -------------------------------------------------------------------- */
/*
    Mid-level access to sparse matrix:

      A = S.Get(i,j)         Get2(i, j, A) (allows missing i,j) 
      A = S.Get(i,j, i2,j2)  S[|i,j \i2,j2|]

      S.Put(i,j,         A)  Put3(i, j, A)  (allows missing i,j)
      S.Put(i, j, i2,j2, A)  S[|i,j \ i2,j2] = A

      S.Replace(...)         same syntax as S.Put()
*/

`RM' `Sp'::Get(one, two, |three, four)
{
      if (args()==2) return(Get2(one, two))
      if (args()==4) return(Get4(one, two, three, four))
      else           _error(3001)   // incorrect # of args
}

void `Sp'::Put(one, two, three, | four, five) 
{

      if      (args()==3)   Put3(one, two, three)
      else if (args()==5)   Put5(one, two, three, four, five)
      else    _error(3001)  // incorrect # of args
}


void `Sp'::Replace(one, two, three, | four, five) 
{

      if      (args()==3) Replace3(one, two, three)
      else if (args()==5) Replace5(one, two, three, four, five)
      else    _error(3001)  // incorrect # of args
}


`RM' `Sp'::Get2(`RS' i, `RS' j)
{
      if (i==. && j==.) return(Get4(1,1, rows, cols))
      if (        j==.) return(Get4(i,1,    i, cols))
      if (i==.)         return(Get4(1,j, rows,    j))
      return(get(i,j))
}

`RM' `Sp'::Get4(`RS' r0, `RS' c0, `RS' r1, `RS' c1)
{
      `RS'     r, c   // all r*, c* vars in this metric
      `RS'     i, j   // i, j            in R[]  metric
      `RM'     R

      if (r0<1    | r0>r1)   _error(3300)  // argument out of range
      if (c0<1    | c0>c1)   _error(3300)
      if (r1>rows | c1>cols) _error(3300)

      R = J(r1-r0+1, c1-c0+1, 0)

      i = 0
      for (r=r0; r<=r1; r++) {
              ++i 
              j = 0 
              for (c=c0; c<=c1; c++) R[i, ++j] = get(r, c) 
      }

      return(R)
}

        
void `Sp'::Put3(`RS' i, `RS' j, `RM' R)
{
      if      (i==. && j==.) Put5(1,1, rows,cols, R)
      else if (        j==.) Put5(i,1,    i,cols, R)
      else if (i==.)         Put5(1,j, rows,   j, R)
      else                   put(i, j, R)
}

void `Sp'::Put5(`RS' r0, `RS' c0, `RS' r1, `RS' c1, `RM' R)
{
      `RS'    i, j
      `RS'    r, c

      i = 0
      for (r=r0; r<=r1; r++) {
              ++i 
              j = 0 
              for (c=c0; c<=c1; c++) put(r, c, R[i,++j])
      }
}


void `Sp'::Replace3(`RS' i, `RS' j, `RM' R)
{
      if (i==. && j==.) Replace5(1,1, rows,cols, R)
      else if (j==.)    Replace5(i,1,    i,cols, R)
      else if (i==.)    Replace5(1,j, rows,   j, R)
      else              replace(i, j, R)
}

void `Sp'::Replace5(`RS' r0, `RS' c0, `RS' r1, `RS' c1, `RM' R)
{
      `RS'    i, j
      `RS'    r, c

      i = 0
      for (r=r0; r<=r1; r++) {
              ++i 
              j = 0 
              for (c=c0; c<=c1; c++) replace(r, c, R[i,++j])
      }
}
        

/* -------------------------------------------------------------------- */
                                        /* Matrix multiplication        */
                                        /* Routines intended for use    */
                                        /* by R_multiply() and          */
                                        /*    S_multiply() only         */

`Regular' `Sp'::R_SxR(`boolean' tA, `Regular' B)
{
      `Regular' R
      `RR'      A_i
      `RS'      rA, cA, rB, cB, i

      // R = A*B where A=this (sparse)

      rA = (tA ? cols : rows) 
      cA = (tA ? rows : cols) 
      rB = (::rows(B))
      cB = (::cols(B))

      if (cA!= rB)  _error(3200)    // conformability error

      R   = J(rA, cB, 0)
      A_i = J( 1, cA, 0)
      for (i=1; i<=rA; i++) {
              set_row(A_i, i, tA)
              R[i, .] = A_i * B
      }
      return(R)
}


`Sparse' `Sp'::S_SxR(`boolean' tA, `Regular' B)
{
      `Sparse'  R
      `RR'      A_i
      `RS'      rA, cA, rB, cB, i

      // R = A*B where A=this (sparse)

      rA = (tA ? cols : rows) 
      cA = (tA ? rows : cols) 
      rB = (::rows(B))
      cB = (::cols(B))

      if (cA!= rB)  _error(3200)    // conformability error

      R.setup(rA, cB)
      A_i = J( 1, cA, 0)
      for (i=1; i<=rA; i++) {
              set_row(A_i, i, tA)
              R.Put(i,., A_i*B)
      }
      return(R)
}


`Regular' `Sp'::R_RxS(`boolean' tB, `Regular' A)
{
      `Regular' R
      `RC'      B_j
      `RS'      rA, cA, rB, cB, j

      // R = A*B where B=this (sparse)

      rA = (::rows(A))
      cA = (::cols(A))
      rB = (tB ? cols : rows)
      cB = (tB ? rows : cols)

      if (cA!= rB)  _error(3200)    // conformability error

      R   = J(rA, cB, 0)
      B_j = J(rB,  1, 0)
      for (j=1; j<=cB; j++) {
              set_col(B_j, j, tB) 
              R[., j] = A * B_j
      }
      return(R)
}


`Sparse' `Sp'::S_RxS(`boolean' tB, `Regular' A)
{
      `Sparse'  R
      `RC'      B_j
      `RS'      rA, cA, rB, cB, j

      // R = A*B where B=this (sparse)

      rA = (::rows(A))
      cA = (::cols(A))
      rB = (tB ? cols : rows)
      cB = (tB ? rows : cols)

      if (cA!= rB)  _error(3200)    // conformability error

      R.setup(rA, cB) 
      B_j = J(rB,  1, 0)
      for (j=1; j<=cB; j++) {
              set_col(B_j, j, tB) 
              R.Put(., j, A*B_j)
      }
      return(R)
}


`Regular' `Sp'::R_SxS(`boolean' tA, `Sparse' B, `boolean' tB)
{ 
      `Regular' R             //         R = A*B 
      `RS'      v             //         v = A[i,h]
      `RR'      subscript     // subscript = (i,h)
      `RS'      r, c, d       // subscript bounds
      `RS'      i, j, h       // subscript indices
      `RS'      k, n
    
      pointer(`RV') vector     p
      pointer(`RV') scalar     q

      // ---------------------------------- code limitation ---
      if (tA | tB) {
              _error(9, "!! tA & tB not yet implemented")
              // NotReached
      }

      // ----------------------------- check conformability ---
      /*
         R_SxS() calculates R=A*B for A, B sparse.
         Notation is
              matrix       dimension    indices
              --------------------------------------
                R            r x c       i, j
                A (this)     r x d       i, h
                B            d x c       h, j
                -------------------------------------
     */

      r = this.rows ; d = this.cols         // A:  r x d
      c = B.rows                            // B:  d x c
                                                    // R:  r x c
      if (d != B.cols) {
              _error(3200)                  // conformability error
              // NotReached
      }

      // ----------------------------------------- multiply --- 
      p = B.cols_of_row(tB) 
      R = J(r, c, 0)
      for (v=AA.firstval(); v; v=AA.nextval()) {
              subscript = AA.key()  
              i         = subscript[1]
              h         = subscript[2] 
              if ( (q=p[h]) ) {
                    n = length(*q)
                    for (k=1; k<=n; k++) {
                            j = (*q)[k]
                            R[i, j] = R[i, j] + v*B.get(h, j)
                    }
              }
      }
      return(R)
}


pointer(`RR') vector `Sp'::cols_of_row(`boolean' tA)
{
      pointer(`RV') vector p          // to be returned
      pointer(`RV') scalar q          // q = p[r] for some r
      `RM'                 subscripts // (row,col) values
      `RS'                 r          // row of matrix
      `RS'                 i, i1, ii  // subscripts for subscripts[]
      `RS'                 k          // subscript  for q[]

      if (tA) _error(9, "!!tA not yet implemented")

      p          = J(this.rows, 1, NULL)
      subscripts = AA.keys()
      if (::rows(subscripts) == 0) return(p) 

      _sort(subscripts, (1,2))

      pragma unset i1
      for (i=1; i<=(::rows(subscripts)); i=i1) {
              r  = subscripts[i, 1]

              // ---------------------------------------------------
              // Indices for row r start at subscripts[   i, .]
              // They continue until        subscripts[i1-1, .]
              // Find i1:

              for (i1=i+1; i1<=(::rows(subscripts)); i1++) {
                      if (subscripts[i1, 1] != r) break
              }

              // ---------------------------------------------------
              // Make a vector containing i1-i elements and
              // store the column subscripts in it:
      
              q = p[r] = &(J(1, i1-i, .))
              k = 0
              for (ii=i; ii<i1; ii++) (*q)[++k] = subscripts[ii,2]
      }
      return(p)
}


`Sparse' `Sp'::S_SxS(`boolean' tA, `Sparse' B, `boolean' tB)
{
      `Regular' R
      `Sparse'  S 
      `RR'      A_i
      `RC'      B_j
      `RS'      rA, cA, rB, cB
      `RS'      i, j

      // R = A*B where AB=this (sparse) and B sparse

      rA = (tA ? this.cols : this.rows) 
      cA = (tA ? this.rows : this.cols) 
      rB = (tB ?    B.cols :    B.rows) 
      cB = (tB ?    B.rows :    B.cols) 


      if (cA != rB)  _error(3200)    // conformability error

      A_i = J( 1, cA, .)
      B_j = J(rB,  1, .)

      S.setup(rA, cB)
      for (i=1; i<=rA; i++) {
              set_row(A_i, i, tA)
              for (j=1; j<=cB; j++) {
                      B.set_col(B_j, j, tB)
                      S.put(i, j, A_i * B_j) 
              }
      }
      return(S)
}


void `Sp'::S_RxR(`Regular' A, `Regular' B)
{
      `RR'         A_i
      `RS'         rA, cA, rB, cB
      `RS'         i, j

      // R = A*B, this=R, A and B regular

      rA = ::rows(A) 
      cA = ::cols(A)
      rB = ::rows(B) 
      cB = ::cols(B)

      if (cA != rB) _error(3200)    // conformability error

      this.setup(rA, cB)
      for (i=1; i<=rA; i++) {
              A_i = A[i, .]
              Put(i, ., A_i * B)
      }
}

void `Sp'::set_row(`RR' row_i, `RS' i, `boolean' transposed)
{
      `RS'  j, cA

      cA = (::cols(row_i))

      if (transposed) {
              for (j=1; j<=cA; j++) row_i[j] = get(j, i)
      }
      else {
              for (j=1; j<=cA; j++) row_i[j] = get(i, j)
      }
}


void `Sp'::set_col(`RC' col_j, `RS' j, `boolean' transposed)
{
      `RS'  i, rA

      rA = (::rows(col_j))

      if (transposed) {
              for (i=1; i<=rA; i++) col_j[i] = get(j, i)
      }
      else {
              for (i=1; i<=rA; i++) col_j[i] = get(i, j)
      }
}


/* -------------------------------------------------------------------- */
                                        /* Matrix Addition              */
                                        /* Routines intended for use    */
                                        /* by R_add() and S_add() only  */

`Regular' `Sp'::R_SpS(`boolean' tA, `Sparse' B, `boolean' tB)
{
      `Regular' R
      `RS'          i, r, c, rB, cB
      `RR'          Arow, Brow

      // R = A + B, A=this and B sparse

      r  = (tA ? this.cols : this.rows) 
      c  = (tA ? this.rows : this.cols) 
      rB = (tB ?    B.cols :    B.rows)
      cB = (tB ?    B.rows :    B.cols)

      if (r!=rB | c!=cB)  _error(3200)  // conformability error

      R     =        J(r, c, 0)
      Arow  = Brow = J(1, c, .) 
      for (i=1; i<=r; i++) {
              this.set_row(Arow, i, tA)
                 B.set_row(Brow, i, tB)
              R[i, .] = Arow + Brow
      }
      return(R)
}

`Regular' `Sp'::R_SpR(`boolean' tA, `Regular' B) 
{
      `Regular' R
      `RS'          i, r, c, rB, cB
      `RR'          Arow

      // R = A + B, A=this sparse, B regular

      r  = (tA ? this.cols : this.rows) 
      c  = (tA ? this.rows : this.cols) 
      rB = (::rows(B))
      cB = (::cols(B))

      if (r!=rB | c!=cB)  _error(3200)  // conformability error

      R     = J(r, c, 0)
      Arow  = J(1, c, .) 
      for (i=1; i<=r; i++) {
              this.set_row(Arow, i, tA)
              R[i, .] = Arow + B[i, .]
      }
      return(R)
}

`Sparse' `Sp'::S_SpS(`boolean' tA, `Sparse' B, `boolean' tB)
{
      `Sparse'  R
      `RS'      i, r, c, rB, cB
      `RR'      Arow, Brow

      // R = A + B, A=this sparse, B sparse 

      r  = (tA ? this.cols : this.rows) 
      c  = (tA ? this.rows : this.cols) 
      rB = (tB ?    B.cols :    B.rows) 
      cB = (tB ?    B.rows :    B.cols) 

      if (r!=rB | c!=cB)  _error(3200)  // conformability error

      R.setup(r, c)        
      Arow = Brow = J(1, c, .) 
      for (i=1; i<=r; i++) {
              this.set_row(Arow, i, tA)
                 B.set_row(Brow, i, tB)
              R.Put(i, ., Arow + Brow) 
      }
      return(R)
}


`Sparse' `Sp'::S_SpR(`boolean' tA, `Regular' B) 
{
      `Sparse'      R
      `RS'          i, r, c, rB, cB
      `RR'          Arow

      // R = A + B, A=this sparse, B sparse 

      r  = (tA ? this.cols : this.rows) 
      c  = (tA ? this.rows : this.cols) 
      rB = (::rows(B))
      cB = (::cols(B))

      if (r!=rB | c!=cB)  _error(3200)  // conformability error

      R.setup(r, c)
      Arow  = J(1, c, .) 
      for (i=1; i<=r; i++) {
              this.set_row(Arow, i, tA)
              R.Put(i, ., Arow + B[i, .])
      }
      return(R)
}

void `Sp'::S_RpR(`Regular' A, `Regular' B)
{
      `Sparse'  S
      `RS'         i, r, c, rB, cB

      // R = A + B, A=this regular, B regular 

      r  = (::rows(A))
      c  = (::cols(A))
      rB = (::rows(B))
      cB = (::cols(B))

      if (r!=rB | c!=cB)  _error(3200)  // conformability error

      S.setup(r, c)
      for (i=1; i<=r; i++) S.Put(i, ., A[i,.] + B[i, .])
      return(S)
}



/* -------------------------------------------------------------------- */
                                /* functions outside the class                */


`Regular' R_multiply(transmorphic A, `boolean' tA,
                     transmorphic B, `boolean' tB)
{
      `boolean'        A_is_Sparse, B_is_Sparse

      A_is_Sparse = Sparse_or_real(A)
      B_is_Sparse = Sparse_or_real(B)

      if (A_is_Sparse) {
              if (B_is_Sparse) return(R_SxS(A, tA, B, tB))
                                 return(R_SxR(A, tA, B, tB))
      }
      else {
              if (B_is_Sparse) return(R_RxS(A, tA, B, tB))
                               return(R_RxR(A, tA, B, tB))
      }
      // NotReached
}

`Sparse' S_multiply(transmorphic A, `boolean' tA,
                    transmorphic B, `boolean' tB)
{
      `boolean'        A_is_Sparse, B_is_Sparse

      A_is_Sparse = Sparse_or_real(A)
      B_is_Sparse = Sparse_or_real(B)

      if (A_is_Sparse) {
              if (B_is_Sparse) return(S_SxS(A, tA, B, tB))
                                return(S_SxR(A, tA, B, tB))
      }
      else {
              if (B_is_Sparse) return(S_RxS(A, tA, B, tB))
                               return(S_RxR(A, tA, B, tB))
      }
      // NotReached
}

`boolean' Sparse_or_real(transmorphic M)
{
      if (classname(M)=="`Sp'") return(`True')
      if (   eltype(M)=="real") return(`False')
      _error(3250)   // type mismatach error
      // NotReached
}


`Regular' R_SxS(`Sparse' A, `boolean' tA, 
                `Sparse' B, `boolean' tB) return(A.R_SxS(tA, B, tB))


`Regular' R_SxR(`Sparse'  A, `boolean' tA, 
                `Regular' B, `boolean' tB)
{
      `Regular' R

      if (tB) _transpose(B)
      R = A.R_SxR(tA, B)
      if (tB) _transpose(B)
      return(R)
}

`Regular' R_RxS(`Regular' A, `boolean' tA, 
                `Sparse'  B, `boolean' tB)
{
      `Regular' R

      if (tA) _transpose(A)
      R = B.R_RxS(tB, A)
      if (tA) _transpose(A)
      return(R)
}

`Regular' R_RxR(`Regular' A, `boolean' tA, 
                `Regular' B, `boolean' tB)
{
      `Regular'  R

      if (tA) _transpose(A)
      if (tB) _transpose(B)
      R = A*B
      if (tB) _transpose(B)
      if (tA) _transpose(A)

      return(R)
}

`Sparse' S_SxS(`Sparse' A, `boolean' tA, 
                `Sparse' B,`boolean' tB) return(A.S_SxS(tA, B, tB))

`Sparse' S_SxR(`Sparse'  A, `boolean' tA, 
               `Regular' B, `boolean' tB)
{
      `Sparse'        S

      if (tB) _transpose(B)
      S = A.S_SxR(tA, B)
      if (tB) _transpose(B)
      return(S)
}

`Sparse' S_RxS(`Regular' A, `boolean' tA, 
               `Sparse'  B, `boolean' tB)
{
      `Sparse' S

      if (tA) _transpose(A)
      S = B.S_RxS(tB, A)
      if (tA) _transpose(A)
      return(S)
}

`Sparse' S_RxR(`Regular' A, `boolean' tA, 
               `Regular' B, `boolean' tB)
{
      `Sparse'    D

      if (tA) _transpose(A)
      if (tB) _transpose(B)
      D.S_RxR(A, B)
      if (tB) _transpose(B)
      if (tA) _transpose(A)
      return(D)
}


/* -------------------------------------------------------------------- */


`Regular' R_add(transmorphic A, `boolean' tA,
                transmorphic B, `boolean' tB)
{
      `boolean'        A_is_Sparse, B_is_Sparse

      A_is_Sparse = Sparse_or_real(A)
      B_is_Sparse = Sparse_or_real(B)

      if (A_is_Sparse) {
              if (B_is_Sparse) return(R_SpS(A, tA, B, tB))
                                return(R_SpR(A, tA, B, tB))
      }
      else {
              if (B_is_Sparse) return(R_SpR(B, tB, A, tA))
                               return(R_RpR(A, tA, B, tB))
      }
      // NotReached
}

`Sparse' S_add(transmorphic A, `boolean' tA,
               transmorphic B, `boolean' tB)
{
      `boolean'        A_is_Sparse, B_is_Sparse

      A_is_Sparse = Sparse_or_real(A)
      B_is_Sparse = Sparse_or_real(B)

      if (A_is_Sparse) {
              if (B_is_Sparse) return(S_SpS(A, tA, B, tB))
                               return(S_SpR(A, tA, B, tB))
      }
      else {
              if (B_is_Sparse) return(S_SpR(B, tB, A, tA))
                               return(S_RpR(A, tA, B, tB))
      }
      // NotReached
}


`RM' R_SpS(`Sparse' A, `boolean' tA, 
           `Sparse' B, `boolean' tB)  return(A.R_SpS(tA, B, tB))

`Regular' R_SpR(`Sparse'  A, `boolean' tA, 
                `Regular' B, `boolean' tB)
{
      `Regular'  R

      if (tB) _transpose(B)
      R = A.R_SpR(tA, B)
      if (tB) _transpose(B)
      return(R)
}


`Regular' R_RpR(`Regular' A, `boolean' tA, 
                `Regular' B, `boolean' tB)
{
      `Regular'  R

      if (tA) _transpose(A)
      if (tB) _transpose(B)
      R = A+B
      if (tB) _transpose(B)
      if (tA) _transpose(A)

      return(R)
}


`Sparse' S_SpS(`Sparse' A, `boolean' tA, 
               `Sparse' B, `boolean' tB)  return(A.S_SpS(tA, B, tB))

`Sparse' S_SpR(`Sparse'  A, `boolean' tA, 
               `Regular' B, `boolean' tB)
{
      `Sparse'    S

      if (tB) _transpose(B)
      S = A.S_SpR(tA, B)
      if (tB) _transpose(B)
      return(S)
}

`Sparse' S_RpR(`Regular' A, `boolean' tA, 
               `Regular' B, `boolean' tB)
{
      `Sparse'  D

      if (tA) _transpose(A)
      if (tB) _transpose(B)
      D.S_RpR(A, B)
      if (tB) _transpose(B)
      if (tA) _transpose(A)
      return(D)
}

end
