// version number intentionally omitted

clear all
run linreg1.mata 

sysuse auto 
keep mpg weight displacement
* keep in 1/10

// ---------------------------------------- test 1, with intercept
reg mpg weight displacement 
predict double yhat

mata:
st_view(X = ., ., ("weight", "displacement"))
st_view(y = ., ., "mpg") 

r = LinReg()
r.setup(y, X, 1)

// coefficients
trub = st_matrix("e(b)")
b = r.b()
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-14)

// ess
truee = st_numscalar("e(rss)")
r.ee(),  truee
mreldif(r.ee(), truee)
assert(mreldif(r.ee(), truee) < 1e-12)

// s2
trus2 = st_numscalar("e(rmse)")^2
r.s2(), trus2
reldif(r.s2(), trus2)
assert( reldif(r.s2(), trus2) < 1e-15)

// V
truV = st_matrix("e(V)") 
r.V()
mreldif(r.V(), truV)
assert(mreldif(r.V(), truV) < 1e-15)

// yhat
st_view(truyhat = ., ., "yhat")
mreldif(r.yhat(), truyhat)
assert(mreldif(r.yhat(), truyhat) <= 1e-15)

end

// ---------------------------------------- test 2, without intercept
reg mpg weight displacement, nocons
drop yhat 
predict double yhat 

mata:
st_view(X = ., ., ("weight", "displacement"))
st_view(y = ., ., "mpg") 

r.setup(y, X, 0)

// coefficients
trub = st_matrix("e(b)")
trub
b = r.b()
b
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-15)

// ess
truee = st_numscalar("e(rss)")
r.ee(),  truee
mreldif(r.ee(), truee)
assert(mreldif(r.ee(), truee) < 1e-15)

// s2
trus2 = st_numscalar("e(rmse)")^2
r.s2(), trus2
reldif(r.s2(), trus2)
assert( reldif(r.s2(), trus2) < 1e-15 )

// V
truV = st_matrix("e(V)") 
r.V()
mreldif(r.V(), truV)
assert(mreldif(r.V(), truV) < 1e-15)

// yhat
st_view(truyhat = ., ., "yhat")
mreldif(r.yhat(), truyhat)
assert(mreldif(r.yhat(), truyhat) <= 1e-13)
end


// ---------------------------------------- test 3, 0 RHS with intercept
reg mpg 
drop yhat 
predict double yhat

mata:
st_view(y = ., ., "mpg") 
X = J(rows(y), 0, 0)		// no rhs vars

r.setup(y, X, 1)

// coefficients
trub = st_matrix("e(b)")
b = r.b()
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-14)

// ess
truee = st_numscalar("e(rss)")
r.ee(),  truee
mreldif(r.ee(), truee)
assert(mreldif(r.ee(), truee) < 1e-15)

// s2
trus2 = st_numscalar("e(rmse)")^2
r.s2(), trus2
reldif(r.s2(), trus2)
assert(reldif(r.s2(), trus2) < 1e-15)

// V
truV = st_matrix("e(V)") 
r.V()
mreldif(r.V(), truV)
assert(mreldif(r.V(), truV) < 1e-15)

// yhat
st_view(truyhat = ., ., "yhat")
mreldif(r.yhat(), truyhat)
assert(mreldif(r.yhat(), truyhat) == 0)
end

// ---------------------------------------- test 4, 0 RHS w/o intercept
reg mpg,  nocons
drop yhat 
predict double yhat 

mata:
st_view(y = ., ., "mpg") 
X = J(rows(y), 0, 0)		// no rhs vars

r.setup(y, X, 0)

// coefficients
trub = st_matrix("e(b)")
assert(cols( trub)==0  & rows( trub)==1)
assert(rows(r.b())==0  & cols(r.b())==1)

// ess
truee = st_numscalar("e(rss)")
r.ee(),  truee
mreldif(r.ee(), truee)
assert(r.ee() == truee) 

// s2
trus2 = st_numscalar("e(rmse)")^2
r.s2(), trus2
reldif(r.s2(), trus2)
assert(r.s2() == trus2) 

// V
truV = st_matrix("e(V)") 
assert(rows( truV)==0 & cols( truV)==0)
assert(rows(r.V())==0 & cols(r.V())==0)

// yhat
st_view(truyhat = ., ., "yhat")
mreldif(r.yhat(), truyhat)
assert(mreldif(r.yhat(), truyhat) == 0)
assert(sum(truyhat :!=0) == 0)
assert(sum(r.yhat():!=0) == 0)
end


// ---------------------------------------- test 5, collinear w/ intercept
reg mpg weight displacement weight
drop yhat
predict double yhat 

mata:
st_view(X = ., ., ("weight", "displacement", "weight"))
st_view(y = ., ., "mpg") 

r.setup(y, X, 1)

// coefficients
trub = st_matrix("e(b)")
b = r.b()
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-15)

// ess
truee = st_numscalar("e(rss)")
r.ee(),  truee
mreldif(r.ee(), truee)
assert(mreldif(r.ee(), truee) < 1e-15)

// s2
trus2 = st_numscalar("e(rmse)")^2
r.s2(), trus2
reldif(r.s2(), trus2)
assert( reldif(r.s2(), trus2) < 1e-15 )

// V
truV = st_matrix("e(V)") 
r.V()
mreldif(r.V(), truV)
assert(mreldif(r.V(), truV) < 1e-15)

// yhat
st_view(truyhat = ., ., "yhat")
mreldif(r.yhat(), truyhat)
assert(mreldif(r.yhat(), truyhat) < 1e-15)
end


// ---------------------------------------- test 6, collinear w/o intercept

reg mpg weight displacement weight, nocons
drop yhat
predict double yhat 

mata:
st_view(X = ., ., ("weight", "displacement", "weight"))
st_view(y = ., ., "mpg") 

r.setup(y, X, 0)

// coefficients
trub = st_matrix("e(b)")
b = r.b()
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-15)

// ess
truee = st_numscalar("e(rss)")
r.ee(),  truee
mreldif(r.ee(), truee)
assert(mreldif(r.ee(), truee) == 0)

// s2
trus2 = st_numscalar("e(rmse)")^2
r.s2(), trus2
reldif(r.s2(), trus2)
assert(r.s2() == trus2)

// V
truV = st_matrix("e(V)") 
r.V()
mreldif(r.V(), truV)
assert(mreldif(r.V(), truV) == 0) 

// yhat
st_view(truyhat = ., ., "yhat")
mreldif(r.yhat(), truyhat)
assert(mreldif(r.yhat(), truyhat) < 1e-13)

end



// ---------------------------------------- test 7, 0 obs w/ intercept

mata:
X = J(0, 3, .)
y = J(0, 1, .) 

r.setup(y, X, 1)

// coefficients
b = r.b()
b'
assert(b' == (.,.,.,.))


// ess
assert(r.ee()==0)

// s2
assert(r.s2()==0)

// V
truV1 = J(3,3,0), J(3,1,0) \ J(1,3,0), .
truV2 = J(3,3,0), J(3,1,.) \ J(1,3,.), .
r.V()
assert(r.V() == truV1 | r.V()==truV2)

// yhat
assert(rows(r.yhat())==0 & cols(r.yhat())==1)
end


// ---------------------------------------- test 8, 0 obs w/o intercept

mata:
X = J(0, 3, .)
y = J(0, 1, .) 

r.setup(y, X, 0)

// coefficients
b = r.b()
b'
assert(b' == (0,0,0))


// ess
assert(r.ee()==0)

// s2
assert(r.s2()==.)

// V
assert(r.V() == J(3,3,.))

// yhat
assert(rows(r.yhat())==0 & cols(r.yhat())==1)
end
