// version number intentionally omitted

clear all
run linreg2.mata 

sysuse auto 
keep mpg weight displacement
* keep in 1/10

// ---------------------------------------- test 1, with intercept
reg mpg weight displacement 
predict double yhat

mata:
st_view(X = ., ., ("weight", "displacement"))
st_view(y = ., ., "mpg") 

r = LinReg()
r.setup(y, X, 1)

// coefficients
trub = st_matrix("e(b)")
b = r.b()
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-14)

// ess
truee = st_numscalar("e(rss)")
r.ee(),  truee
mreldif(r.ee(), truee)
assert(mreldif(r.ee(), truee) < 1e-12)

// s2
trus2 = st_numscalar("e(rmse)")^2
r.s2(), trus2
reldif(r.s2(), trus2)
assert( reldif(r.s2(), trus2) < 1e-15)

// V
truV = st_matrix("e(V)") 
r.V()
mreldif(r.V(), truV)
assert(mreldif(r.V(), truV) < 1e-15)
assert(issymmetric(r.V()))

// yhat
st_view(truyhat = ., ., "yhat")
mreldif(r.yhat(), truyhat)
assert(mreldif(r.yhat(), truyhat) <= 1e-15)

end

// ---------------------------------------- test 2, without intercept
reg mpg weight displacement, nocons
drop yhat 
predict double yhat 

mata:
st_view(X = ., ., ("weight", "displacement"))
st_view(y = ., ., "mpg") 

r.setup(y, X, 0)

// coefficients
trub = st_matrix("e(b)")
trub
b = r.b()
b
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-15)

// ess
truee = st_numscalar("e(rss)")
r.ee(),  truee
mreldif(r.ee(), truee)
assert(mreldif(r.ee(), truee) < 1e-15)

// s2
trus2 = st_numscalar("e(rmse)")^2
r.s2(), trus2
reldif(r.s2(), trus2)
assert( reldif(r.s2(), trus2) < 1e-15 )

// V
truV = st_matrix("e(V)") 
r.V()
mreldif(r.V(), truV)
assert(mreldif(r.V(), truV) < 1e-15)
assert(issymmetric(r.V()))

// yhat
st_view(truyhat = ., ., "yhat")
mreldif(r.yhat(), truyhat)
assert(mreldif(r.yhat(), truyhat) <= 1e-13)
end


// ---------------------------------------- test 3, 0 RHS with intercept
reg mpg 
drop yhat 
predict double yhat

mata:
st_view(y = ., ., "mpg") 
X = J(rows(y), 0, 0)                // no rhs vars

r.setup(y, X, 1)

// coefficients
trub = st_matrix("e(b)")
b = r.b()
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-14)

// ess
truee = st_numscalar("e(rss)")
r.ee(),  truee
mreldif(r.ee(), truee)
assert(mreldif(r.ee(), truee) < 1e-15)

// s2
trus2 = st_numscalar("e(rmse)")^2
r.s2(), trus2
reldif(r.s2(), trus2)
assert(reldif(r.s2(), trus2) < 1e-15)

// V
truV = st_matrix("e(V)") 
r.V()
mreldif(r.V(), truV)
assert(mreldif(r.V(), truV) < 1e-15)
assert(issymmetric(r.V()))

// yhat
st_view(truyhat = ., ., "yhat")
mreldif(r.yhat(), truyhat)
assert(mreldif(r.yhat(), truyhat) == 0)
end

// Vrobust
reg mpg, robust
mata:
truV = st_matrix("e(V)") 
r.Vrobust()
mreldif(r.Vrobust(), truV)
assert(mreldif(r.Vrobust(), truV) < 1e-15)
assert(issymmetric(r.Vrobust()))
end

// Vopg
glm mpg, vce(opg)
mata:
truV = st_matrix("e(V)") 
r.Vrobust()
mreldif(r.Vopg(), truV)
assert(mreldif(r.Vopg(), truV) < 1e-15)
assert(issymmetric(r.Vopg()))
end


// ---------------------------------------- test 4, 0 RHS w/o intercept
reg mpg,  nocons
drop yhat 
predict double yhat 

mata:
st_view(y = ., ., "mpg") 
X = J(rows(y), 0, 0)                // no rhs vars

r.setup(y, X, 0)

// coefficients
trub = st_matrix("e(b)")
assert(cols( trub)==0  & rows( trub)==1)
assert(rows(r.b())==0  & cols(r.b())==1)

// ess
truee = st_numscalar("e(rss)")
r.ee(),  truee
mreldif(r.ee(), truee)
assert(r.ee() == truee) 

// s2
trus2 = st_numscalar("e(rmse)")^2
r.s2(), trus2
reldif(r.s2(), trus2)
assert(r.s2() == trus2) 

// V
truV = st_matrix("e(V)") 
assert(rows( truV)==0 & cols( truV)==0)
assert(rows(r.V())==0 & cols(r.V())==0)
assert(issymmetric(r.V()))

// yhat
st_view(truyhat = ., ., "yhat")
mreldif(r.yhat(), truyhat)
assert(mreldif(r.yhat(), truyhat) == 0)
assert(sum(truyhat :!=0) == 0)
assert(sum(r.yhat():!=0) == 0)
end

// Vrobust
reg mpg,  nocons vce(robust)
mata:
truV = st_matrix("e(V)") 
assert(rows( truV)==0 & cols( truV)==0)
assert(rows(r.Vrobust())==0 & cols(r.Vrobust())==0)
assert(issymmetric(r.Vrobust()))
end

// Vopg
rcof "noisily glm mpg, vce(opg) nocons" == 100
mata:
assert(rows(r.Vopg())==0 & cols(r.Vopg())==0)
assert(issymmetric(r.Vopg()))
end


// ---------------------------------------- test 5, collinear w/ intercept
reg mpg weight displacement weight
drop yhat
predict double yhat 

mata:
st_view(X = ., ., ("weight", "displacement", "weight"))
st_view(y = ., ., "mpg") 

r.setup(y, X, 1)

// coefficients
trub = st_matrix("e(b)")
b = r.b()
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-15)

// ess
truee = st_numscalar("e(rss)")
r.ee(),  truee
mreldif(r.ee(), truee)
assert(mreldif(r.ee(), truee) < 1e-15)

// s2
trus2 = st_numscalar("e(rmse)")^2
r.s2(), trus2
reldif(r.s2(), trus2)
assert( reldif(r.s2(), trus2) < 1e-15 )

// V
truV = st_matrix("e(V)") 
r.V()
mreldif(r.V(), truV)
assert(mreldif(r.V(), truV) < 1e-15)
assert(issymmetric(r.V()))

// yhat
st_view(truyhat = ., ., "yhat")
mreldif(r.yhat(), truyhat)
assert(mreldif(r.yhat(), truyhat) < 1e-15)
end

// Vrobust
reg mpg weight displacement weight, vce(robust)
mata:
truV = st_matrix("e(V)") 
r.Vrobust()
mreldif(r.Vrobust(), truV)
assert(mreldif(r.Vrobust(), truV) < 1e-14)
assert(issymmetric(r.Vrobust()))
end

// Vopg
glm mpg weight displacement weight, vce(opg)
mata:
truV = st_matrix("e(V)") 
r.Vopg()
mreldif(r.Vopg(), truV)
assert(mreldif(r.Vopg(), truV) < 1e-13)
assert(issymmetric(r.Vopg()))
end





// ---------------------------------------- test 6, collinear w/o intercept

reg mpg weight displacement weight, nocons
drop yhat
predict double yhat 

mata:
st_view(X = ., ., ("weight", "displacement", "weight"))
st_view(y = ., ., "mpg") 

r.setup(y, X, 0)

// coefficients
trub = st_matrix("e(b)")
b = r.b()
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-15)

// ess
truee = st_numscalar("e(rss)")
r.ee(),  truee
mreldif(r.ee(), truee)
assert(mreldif(r.ee(), truee) == 0)

// s2
trus2 = st_numscalar("e(rmse)")^2
r.s2(), trus2
reldif(r.s2(), trus2)
assert(r.s2() == trus2)

// V
truV = st_matrix("e(V)") 
r.V()
mreldif(r.V(), truV)
assert(mreldif(r.V(), truV) == 0) 

// yhat
st_view(truyhat = ., ., "yhat")
mreldif(r.yhat(), truyhat)
assert(mreldif(r.yhat(), truyhat) < 1e-13)
end

// Vrobust
reg mpg weight displacement weight, nocons vce(robust)
mata:
truV = st_matrix("e(V)") 
r.Vrobust()
mreldif(r.Vrobust(), truV)
assert(mreldif(r.Vrobust(), truV) < 1e-17) 
assert(issymmetric(r.Vrobust()))
end


// Vopg
glm mpg weight displacement weight, nocons vce(opg)
mata:
truV = st_matrix("e(V)") 
r.Vopg()
mreldif(r.Vopg(), truV)
assert(mreldif(r.Vopg(), truV) < 1e-17) 
assert(issymmetric(r.Vopg()))
end



// ---------------------------------------- test 7, 0 obs w/ intercept

mata:
X = J(0, 3, .)
y = J(0, 1, .) 

r.setup(y, X, 1)

// coefficients
b = r.b()
b'
assert(b' == (.,.,.,.))


// ess
assert(r.ee()==0)

// s2
assert(r.s2()==0)

// V
truV = J(3,3,0), J(3,1,.) \ J(1,3,.), .
r.V()
assert(r.V() == truV)
assert(issymmetric(r.V()))

// Vrobust
truV = /* J(3,3,0), J(3,1,.) \ J(1,3,.), . */ J(4,4,.)
r.Vrobust()
assert(r.Vrobust() == truV)
assert(issymmetric(r.Vrobust()))

// opg
r.Vopg()
assert(r.Vopg() == J(4,4,.))
assert(issymmetric(r.Vopg()))

// yhat
assert(rows(r.yhat())==0 & cols(r.yhat())==1)
end


// ---------------------------------------- test 8, 0 obs w/o intercept

mata:
X = J(0, 3, .)
y = J(0, 1, .) 

r.setup(y, X, 0)

// coefficients
b = r.b()
b'
assert(b' == (0,0,0))


// ess
assert(r.ee()==0)

// s2
assert(r.s2()==.)

// V
assert(r.V() == J(3,3,.))
assert(issymmetric(r.V()))

// Vrobust
assert(r.Vrobust() == J(3,3,.))
assert(issymmetric(r.Vrobust()))

// Vopg
assert(r.Vopg() == J(3,3,.))
assert(issymmetric(r.Vopg()))

// yhat
assert(rows(r.yhat())==0 & cols(r.yhat())==1)
end


// ---------------------------------------- test 9, opg w/ intercept
glm mpg weight displacement, vce(opg) 
mata:
st_view(y = ., ., "mpg")
st_view(X = ., ., ("weight", "displacement"))

r.setup(y, X, 1) 
assert(mreldif(r.b()', st_matrix("e(b)")) < 1e-15)
r.Vopg() 
r.Vopg() :/ st_matrix("e(V)")
mreldif(r.Vopg(), st_matrix("e(V)")) 
assert(mreldif(r.Vopg(), st_matrix("e(V)")) < 1e-13)
assert(issymmetric(r.Vopg()))
end

// ---------------------------------------- test 10, robust w/ intercept
regress mpg weight displacement, vce(robust)
mata:
r.Vrobust()
r.Vrobust() :/ st_matrix("e(V)")
mreldif(r.Vrobust(), st_matrix("e(V)"))
assert (mreldif(r.Vrobust(), st_matrix("e(V)")) < 1e-14)
assert(issymmetric(r.Vrobust()))
end



// ---------------------------------------- test 11, opg w/o intercept
glm mpg weight displacement, vce(opg) nocons
mata:
st_view(y = ., ., "mpg")
st_view(X = ., ., ("weight", "displacement"))

r.setup(y, X, 0) 
assert(mreldif(r.b()', st_matrix("e(b)")) < 1e-15)
r.Vopg() 
r.Vopg() :/ st_matrix("e(V)")
mreldif(r.Vopg(), st_matrix("e(V)"))
assert(mreldif(r.Vopg(), st_matrix("e(V)")) < 1e-17)
assert(issymmetric(r.Vopg()))
end


// ---------------------------------------- test 12, robust w/ intercept
regress mpg weight displacement, vce(robust) nocons
mata:
r.Vrobust()
r.Vrobust() :/ st_matrix("e(V)")
mreldif(r.Vrobust(), st_matrix("e(V)"))
assert (mreldif(r.Vrobust(), st_matrix("e(V)")) < 1e-14)
assert(issymmetric(r.Vrobust()))
end
