// version number intentionally omitted

clear all
run lr2.mata

sysuse auto 
keep mpg weight displacement

// -----------------------------------------------------------
// Utility routines to test "equality".
mata:
void virtuallyequal(real matrix A, real matrix B)
{
        almostequal(A, B, 1e-15)
}

void almostequal(real matrix A, real matrix B, real scalar tol)
{
        assert(mreldif(A, B) <= tol)
}
end

// Prove that almostequal() works:
mata:
b1 = (1, 1, 1)
b2 = (1, 1, 1+1e-14)
end
capture noisily mata: virtuallyequal(b1, b2)
assert _rc!=0

// ---------------------------------------------------------------
// Test 1:  Model with an intercept.

regress mpg weight displacement 
predict double yhat

mata:
st_view(X = ., ., ("weight", "displacement"))
st_view(y = ., ., "mpg") 
r = lrset(y, X, 1)
virtuallyequal(lr_b(r)',   st_matrix("e(b)"))
virtuallyequal(lr_ee(r),   st_numscalar("e(rss)"))
virtuallyequal(lr_s2(r),   st_numscalar("e(rmse)")^2)
virtuallyequal(lr_V(r),    st_matrix("e(V)"))
virtuallyequal(lr_yhat(r), st_data(., "yhat"))
end

// ---------------------------------------------------------------
// Test 2:  Model without an intercept.

reg mpg weight displacement, nocons
drop yhat 
predict double yhat 
mata:
st_view(X = ., ., ("weight", "displacement"))
st_view(y = ., ., "mpg") 
r = lrset(y, X, 0)
virtuallyequal(lr_b(r)',   st_matrix("e(b)"))
virtuallyequal(lr_ee(r),   st_numscalar("e(rss)"))
virtuallyequal(lr_s2(r),   st_numscalar("e(rmse)")^2)
virtuallyequal(lr_V(r),    st_matrix("e(V)"))
almostequal(   lr_yhat(r), st_data(., "yhat"), 1e-13)
end

// ---------------------------------------------------------------
// Test 3:  Zero RHS vars, with intercept.

reg mpg 
drop yhat 
predict double yhat

mata:
st_view(y = ., ., "mpg") 
X = J(rows(y), 0, 0)                // no RHS vars
r = lrset(y, X, 1)
virtuallyequal(lr_b(r)',   st_matrix("e(b)"))
virtuallyequal(lr_ee(r),   st_numscalar("e(rss)"))
virtuallyequal(lr_s2(r),   st_numscalar("e(rmse)")^2)
virtuallyequal(lr_V(r),    st_matrix("e(V)"))
virtuallyequal(lr_yhat(r), st_data(., "yhat"))
end

// ---------------------------------------------------------------
// Test 4:  Zero RHS vars, without an intercept.

reg mpg,  nocons
drop yhat 
predict double yhat 

mata:
st_view(y = ., ., "mpg") 
X = J(rows(y), 0, 0)                // no RHS vars

r = lrset(y, X, 0)
virtuallyequal(lr_b(r)',   st_matrix("e(b)"))
virtuallyequal(lr_ee(r),   st_numscalar("e(rss)"))
virtuallyequal(lr_s2(r),   st_numscalar("e(rmse)")^2)
virtuallyequal(lr_V(r),    st_matrix("e(V)"))
virtuallyequal(lr_yhat(r), st_data(., "yhat"))
end

// ---------------------------------------------------------------
// Test 5:  Collinear RHS var, with intercept.

reg mpg weight displacement weight
drop yhat
predict double yhat 

mata:
st_view(X = ., ., ("weight", "displacement", "weight"))
st_view(y = ., ., "mpg") 

r = lrset(y, X, 1)
virtuallyequal(lr_b(r)',   st_matrix("e(b)"))
virtuallyequal(lr_ee(r),   st_numscalar("e(rss)"))
virtuallyequal(lr_s2(r),   st_numscalar("e(rmse)")^2)
virtuallyequal(lr_V(r),    st_matrix("e(V)"))
virtuallyequal(lr_yhat(r), st_data(., "yhat"))
end

// ---------------------------------------------------------------
// Test 6:  Collinear RHS var, without an intercept.

reg mpg weight displacement weight, nocons
drop yhat
predict double yhat 

mata:
st_view(X = ., ., ("weight", "displacement", "weight"))
st_view(y = ., ., "mpg") 

r = lrset(y, X, 0)
virtuallyequal(lr_b(r)',   st_matrix("e(b)"))
virtuallyequal(lr_ee(r),   st_numscalar("e(rss)"))
virtuallyequal(lr_s2(r),   st_numscalar("e(rmse)")^2)
virtuallyequal(lr_V(r),    st_matrix("e(V)"))
almostequal(   lr_yhat(r), st_data(., "yhat"), 1e-13)
end

// ---------------------------------------------------------------
// Test 7:  Zero observations, with intercept.

mata:
X = J(0, 3, .)
y = J(0, 1, .) 

r = lrset(y, X, 1)
assert(lr_b(r)' == J(1, 4, .))
assert(lr_ee(r) == 0)
assert(lr_s2(r) == 0)
assert(lr_V(r)  == (J(3,3,0), J(3,1,.) \ J(1,4,.)))   // yuck!
assert(rows(lr_yhat(r))==0 & cols(lr_yhat(r))==1)
end

// ---------------------------------------------------------------
// Test 8:  Zero observations, without an intercept.

mata:
X = J(0, 3, .)
y = J(0, 1, .) 

r = lrset(y, X, 0)

// Coefficients.
assert(lr_b(r)' == (0,0,0))
assert(lr_ee(r) == 0)
assert(lr_s2(r) == .)
assert(lr_V(r)  ==  J(3,3,.))
assert(rows(lr_yhat(r))==0 & cols(lr_yhat(r))==1)
end
