// version number intentionally omitted

clear all 
run spmat1.mata

local N 7

mata:
void post(class SpMat S, transmorphic B)
{
        real scalar        i, j

        S.setup(rows(B), cols(B))
        for (i=1; i<=rows(B); i++) {
                for (j=1; j<=cols(B); j++) S.put(i, j, B[i,j])
        }
}

void assertequal(class SpMat S, real matrix B)
{
        real scalar        i, j

        assert(S.rows() == rows(B))
        assert(S.cols() == cols(B))
        "row/cols verified"
        
        for (i=1; i<=rows(B); i++) {
                for (j=1; j<=cols(B); j++) assert(S.get(i, j) == B[i,j])
        }
        "values verified"
        
}

end

set seed 399948

mata:
truS = diag((1..`N'))
for (i=1; i<`N'; i++) truS[  i,i+1] = i - 5
for (i=1; i<`N'; i++) truS[i+1,  i] = i - 9
truS = truS \ truS[4, .]
assert(rows(truS)==`N'+1 & cols(truS)==`N')
// truS is not square. 


S = SpMat()
post(S, truS)
assertequal(S, truS)

copyS = S.Get(., .)
assert(copyS==truS) 
assert(S.Get(1,2) == -4)
assert(S.Get(1,.) == truS[1, .])
assert(S.Get(.,1) == truS[., 1])
assert(S.Get(3,.) == truS[3, .])
assert(S.Get(.,3) == truS[., 3])
assert(S.Get(1,1, 3,3) == truS[|1,1\3,3|])
assert(S.Get(2,3, 5,4) == truS[|2,3\5,4|])

truB = floor(100*runiform(`N', 3))
assert(S.R_SxR(0, truB) == truS*truB)
end
capture noi mata: assert(S.R_SxR(1, truB) == truS'truB)
assert _rc==3200
mata:
Sprime = SpMat()
Sprime.setup(cols(truS), rows(truS))
Sprime.Put(., ., truS')
assert(Sprime.Get(.,.)==truS')
assert(Sprime.R_SxR(1, truB) == truS*truB)


mata:
void test1(class SpMat scalar S, 
           class SpMat scalar Sprime, truB, truS)

{
        class SpMat scalar T

        T = S.S_SxR(0, truB)
        assert(T.Get(.,.) == truS*truB)
        "untransposed okay"

        T = Sprime.S_SxR(1,truB)
        assert(T.Get(.,.) == truS*truB)
        "transposed okay"
}
test1(S, Sprime, truB, truS)
mata drop test1()

rows(truB), cols(truB)
B = SpMat()
B.setup(rows(truB), cols(truB))
B.Put(., ., truB)
assert(B.Get(.,.)==truB)
Bprime = SpMat()
Bprime.setup(cols(truB), rows(truB))
Bprime.Put(., ., truB')
assert(Bprime.Get(.,.)==truB')

void test2(class SpMat scalar S, 
           class SpMat scalar Sprime, 
           class SpMat scalar B, 
           class SpMat scalar Bprime,
           truB, truS)

{
        class SpMat scalar T

        T = S.S_SxS(0, B, 0)
        assert(T.Get(.,.) == truS*truB)
        "untransposed okay"

        T = Sprime.S_SxS(1, B, 0)
        assert(T.Get(.,.) == truS*truB)
        "transposed okay"

        T = Sprime.S_SxS(1, Bprime, 1)
        assert(T.Get(.,.) == truS*truB)
        "transposed transposed okay"
}

test2(S, Sprime, B, Bprime, truB, truS)
mata drop test2()




void test3(class SpMat scalar S, 
           class SpMat scalar Sprime, 
           class SpMat scalar B, 
           class SpMat scalar Bprime,
           truB, truS)

{
        class SpMat scalar T
        real matrix        R

        printf("truS:  %g x %g\n", (rows(truS)), (cols(truS)))
        printf("truB:  %g x %g\n", (rows(truB)), (cols(truB)))

        R = B.R_RxS(0, truS)
        assert(R==truS*truB)
        "R_RxS(0, ..) okay"


        R = Bprime.R_RxS(1, truS)
        assert(R==truS*truB)
        "R_RxS(1, ..) okay"

        T = B.S_RxS(0, truS)
        assert(T.Get(.,.)==truS*truB)
        "S_RxS(0, ..) okay"

        T = Bprime.S_RxS(1, truS)
        assert(T.Get(.,.)==truS*truB)
        "S_RxS(1, ..) okay"
}

test3(S, Sprime, B, Bprime, truB, truS)
mata drop test3()


// test R_multiply() interface 
assert(R_multiply(S,      0, truB,   0) == truS*truB)
assert(R_multiply(Sprime, 1, truB,   0) == truS*truB)
assert(R_multiply(S,      0, truB',  1) == truS*truB)
assert(R_multiply(Sprime, 1, truB',  1) == truS*truB)

assert(R_multiply(truS,   0, B,      0) == truS*truB)
assert(R_multiply(truS',  1, B,      0) == truS*truB)
assert(R_multiply(truS,   0, Bprime, 1) == truS*truB)
assert(R_multiply(truS',  1, Bprime, 1) == truS*truB)

assert(R_multiply(S,      0, B,      0) == truS*truB)
assert(R_multiply(Sprime, 1, B,      0) == truS*truB)
assert(R_multiply(S,      0, Bprime, 1) == truS*truB)
assert(R_multiply(Sprime, 1, Bprime, 1) == truS*truB)

assert(R_multiply(truS,   0, truB,   0) == truS*truB)
assert(R_multiply(truS',  1, truB,   0) == truS*truB)
assert(R_multiply(truS,   0, truB',  1) == truS*truB)
assert(R_multiply(truS',  1, truB',  1) == truS*truB)


// test S_multiply() interface 

real matrix test4(a, b, c, d)
{
        class SpMat scalar   T

        T = S_multiply(a, b, c, d)
        return(T.Get(.,.))
}

assert(test4(S,      0, truB,   0) == truS*truB)
assert(test4(Sprime, 1, truB,   0) == truS*truB)
assert(test4(S,      0, truB',  1) == truS*truB)
assert(test4(Sprime, 1, truB',  1) == truS*truB)

assert(test4(truS,   0, B,      0) == truS*truB)
assert(test4(truS',  1, B,      0) == truS*truB)
assert(test4(truS,   0, Bprime, 1) == truS*truB)
assert(test4(truS',  1, Bprime, 1) == truS*truB)

assert(test4(S,      0, B,      0) == truS*truB)
assert(test4(Sprime, 1, B,      0) == truS*truB)
assert(test4(S,      0, Bprime, 1) == truS*truB)
assert(test4(Sprime, 1, Bprime, 1) == truS*truB)

assert(test4(truS,   0, truB,   0) == truS*truB)
assert(test4(truS',  1, truB,   0) == truS*truB)
assert(test4(truS,   0, truB',  1) == truS*truB)
assert(test4(truS',  1, truB',  1) == truS*truB)

mata drop test4()



// test R_add() interface 


truC = floor(100*runiform(rows(truS), cols(truS)))
C = SpMat()
C.setup(rows(truS), cols(truS))
C.Put(., ., truC)
assert(C.Get(.,.) == truC)
Cprime = SpMat()
Cprime.setup(cols(truC), rows(truC))
Cprime.Put(., ., truC')
assert(Cprime.Get(.,.) == truC')

assert(R_add(S,      0, truC,   0) == truS+truC)
assert(R_add(Sprime, 1, truC,   0) == truS+truC)
assert(R_add(S,      0, truC',  1) == truS+truC)
assert(R_add(Sprime, 1, truC',  1) == truS+truC)

assert(R_add(truS,   0, C,      0) == truS+truC)
assert(R_add(truS',  1, C,      0) == truS+truC)
assert(R_add(truS,   0, Cprime, 1) == truS+truC)
assert(R_add(truS',  1, Cprime, 1) == truS+truC)

assert(R_add(S,      0, C,      0) == truS+truC)
assert(R_add(Sprime, 1, C,      0) == truS+truC)
assert(R_add(S,      0, Cprime, 1) == truS+truC)
assert(R_add(Sprime, 1, Cprime, 1) == truS+truC)

assert(R_add(truS,   0, truC,   0) == truS+truC)
assert(R_add(truS',  1, truC,   0) == truS+truC)
assert(R_add(truS,   0, truC',  1) == truS+truC)
assert(R_add(truS',  1, truC',  1) == truS+truC)

end
